{% extends "dynamic_preferences/base.html" %}

{% block content %}
    <form action="" enctype="multipart/form-data" method="post">
    {% csrf_token %}
    {{ form.as_p }}
    <input type="submit" value="Submit" />
</form>
{% endblock %}